defmodule Nulla.Relations.Relation do
  use Ecto.Schema
  import Ecto.Changeset
  alias Nulla.Snowflake
  alias Nulla.Actors.Actor

  @primary_key {:id, :integer, autogenerate: false}
  schema "relations" do
    field :following, :boolean, default: false
    field :followed_by, :boolean, default: false
    field :showing_replies, :boolean, default: true
    field :showings_reblogs, :boolean, default: true
    field :notifying, :boolean, default: false
    field :muting, :boolean, default: false
    field :muting_notifications, :boolean, default: false
    field :blocking, :boolean, default: false
    field :blocked_by, :boolean, default: false
    field :domain_blocking, :boolean, default: false
    field :requested, :boolean, default: false
    field :note, :string

    belongs_to :local_actor, Actor
    belongs_to :remote_actor, Actor

    timestamps(type: :utc_datetime)
  end

  @doc false
  def changeset(relation, attrs) do
    relation
    |> cast(attrs, [
      :following,
      :followed_by,
      :showing_replies,
      :showings_reblogs,
      :notifying,
      :muting,
      :muting_notifications,
      :blocking,
      :blocked_by,
      :domain_blocking,
      :requested,
      :note,
      :local_actor_id,
      :remote_actor_id
    ])
    |> maybe_put_id()
    |> validate_required([
      :following,
      :followed_by,
      :showing_replies,
      :showings_reblogs,
      :notifying,
      :muting,
      :muting_notifications,
      :blocking,
      :blocked_by,
      :domain_blocking,
      :requested,
      :note,
      :local_actor_id,
      :remote_actor_id
    ])
    |> unique_constraint([:local_actor_id, :remote_actor_id])
  end

  defp maybe_put_id(changeset) do
    id_in_attrs = get_field(changeset, :id)

    if is_nil(id_in_attrs) do
      change(changeset, id: Snowflake.next_id())
    else
      changeset
    end
  end
end
